# /test/test_setuptools_green.py
#
# Tests for setuptools-green.
#
# See /LICENCE.md for Copyright information
"""Tests for setuptools-green."""
import subprocess

import sys

from distutils.errors import DistutilsArgError

from mock import Mock

from setuptools import Distribution

from setuptools_green import GreenTestCommand

from testtools import ExpectedException, TestCase
from testtools.matchers import (Contains, ContainsAll, Not)


def _subprocess_call_args():
    """Return arguments to last call to subprocess.call."""
    return subprocess.call.call_args[0][0]


class TestGreenTestCommand(TestCase):

    """Test cases for the GreenTestCommand class."""

    def __init__(self, *args, **kwargs):
        """Initialize instance variables on this test case."""
        super(TestGreenTestCommand, self).__init__(*args, **kwargs)

    def setUp(self):  # suppress(N802)
        """Patch out functions to monitor GreenTestCommand behavior."""
        super(TestGreenTestCommand, self).setUp()

        self.patch(subprocess, "call", Mock(spec=subprocess.call))
        self.patch(sys, "exit", Mock(spec=sys.exit))

    def test_run_quiet(self):
        """Run green tests quietly."""
        cmd = GreenTestCommand(Distribution())
        cmd.quiet = True
        cmd.ensure_finalized()
        cmd.run()
        self.assertThat(_subprocess_call_args(),
                        Not(Contains("-vvv")))

    def test_run_concurrently(self):
        """Run green tests concurrently."""
        cmd = GreenTestCommand(Distribution())
        cmd.concurrent = True
        cmd.ensure_finalized()
        cmd.run()
        self.assertThat(_subprocess_call_args(),
                        ContainsAll(["-s", "0"]))

    def test_run_coverage(self):
        """Run green tests with coverage."""
        cmd = GreenTestCommand(Distribution())
        cmd.coverage = True
        cmd.ensure_finalized()
        cmd.run()
        self.assertThat(_subprocess_call_args(), Contains("-r"))

    def test_run_target(self):
        """Run green tests against a predetermined target."""
        cmd = GreenTestCommand(Distribution())
        cmd.target = "test"
        cmd.ensure_finalized()
        cmd.run()
        self.assertThat(_subprocess_call_args(),
                        Contains("test"))

    def test_run_coverage_omit(self):
        """Run green tests against a predetermined target."""
        cmd = GreenTestCommand(Distribution())
        cmd.coverage_omit = "abc/*,*/def"
        cmd.ensure_finalized()
        cmd.run()
        self.assertThat(_subprocess_call_args(),
                        ContainsAll([
                            "-o",
                            cmd.coverage_omit
                        ]))

    def test_invalid_quiet_option(self):  # suppress(no-self-use)
        """Invalidly formed quiet option throws."""
        with ExpectedException(DistutilsArgError):
            cmd = GreenTestCommand(Distribution())
            cmd.quiet = "A string"
            cmd.ensure_finalized()
            cmd.run()

    def test_invalid_target_option(self):  # suppress(no-self-use)
        """Invalidly formed target option throws."""
        with ExpectedException(DistutilsArgError):
            cmd = GreenTestCommand(Distribution())
            cmd.target = True
            cmd.ensure_finalized()
            cmd.run()

    def test_run_verbose(self):
        """Run green tests verbosely."""
        GreenTestCommand(Distribution()).run()
        self.assertThat(_subprocess_call_args(), Contains("-vvv"))

    def test_use_exit_status(self):  # suppress(no-self-use)
        """Use Green's exit status."""
        subprocess.call.return_value = 1
        GreenTestCommand(Distribution()).run()
        sys.exit.assert_called_with(1)



# /setuptools_green/__init__.py
#
# Provides a setuptools command for running tests using the "green"
# test runner.
#
# See /LICENCE.md for Copyright information
"""Provide a setuptools command for running tests with green."""

import subprocess

import sys

from distutils.errors import DistutilsArgError

import setuptools


class GreenTestCommand(setuptools.Command):

    """Provide a test command using green."""

    def __init__(self, *args, **kwargs):
        """Initialize instance variables."""
        setuptools.Command.__init__(self, *args, **kwargs)
        self.quiet = False
        self.concurrent = False
        self.coverage = False
        self.coverage_omit = None
        self.target = None

    def run(self):   # suppress(unused-function)
        """Run tests using green."""
        argv = ["green", "-t"]

        if self.target:
            argv.append(self.target)

        if self.concurrent:
            argv.extend(["-s", "0"])

        if self.coverage:
            argv.append("-r")

        if self.coverage_omit:
            argv.extend(["-o", self.coverage_omit])

        if not self.quiet:
            argv.append("-vvv")

        sys.exit(subprocess.call(argv))

    def initialize_options(self):  # suppress(unused-function)
        """Set all options to their initial values."""
        self.quiet = False
        self.concurrent = False
        self.coverage = False
        self.coverage_omit = None
        self.target = None

    def finalize_options(self):  # suppress(unused-function)
        """Finalize options."""
        for arg in ("concurrent", "coverage", "quiet"):
            if not isinstance(getattr(self, arg), int):
                raise DistutilsArgError("""--{} takes no additional """
                                        """arguments.""".format(arg))

        for arg in ("target", "coverage-omit"):
            if not (isinstance(getattr(self, arg.replace("-", "_")), str) or
                    getattr(self, arg.replace("-", "_")) is None):
                upper_arg = arg.upper()
                raise DistutilsArgError("""--{l}={u}: {u} must be a """
                                        """string.""".format(l=arg,
                                                             u=upper_arg))

    user_options = [  # suppress(unused-variable)
        ("quiet", None, """Don't show test descriptions when running"""),
        ("concurrent", None, """Run tests concurrently"""),
        ("coverage", None, """Collect coverage"""),
        ("coverage-omit=", None, ("""Patterns to omit from coverage, """
                                  """comma separated""")),
        ("target=", None, ("""Name of subdirectory where tests are to be """
                           """found"""))
    ]
    # suppress(unused-variable)
    description = "run tests using the 'green' test runner"



# /setup.py
#
# Installation and setup script for setuptools-green
#
# See /LICENCE.md for Copyright information
"""Installation and setup script for setuptools-green."""

from setuptools import find_packages
from setuptools import setup

try:
    from setuptools_green import GreenTestCommand
    _CMD_CLASSES = {
        "green": GreenTestCommand
    }
except ImportError:
    _CMD_CLASSES = dict()

setup(name="setuptools-green",
      version="0.0.13",
      description="Provides a 'test' command for running tests with green",
      long_description_markdown_filename="README.md",
      author="Sam Spilsbury",
      author_email="smspillaz@gmail.com",
      url="http://github.com/polysquare/setuptools-green",
      classifiers=["Development Status :: 3 - Alpha",
                   "Programming Language :: Python :: 2",
                   "Programming Language :: Python :: 2.7",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.1",
                   "Programming Language :: Python :: 3.2",
                   "Programming Language :: Python :: 3.3",
                   "Programming Language :: Python :: 3.4",
                   "Intended Audience :: Developers",
                   "Topic :: Software Development :: Build Tools",
                   "License :: OSI Approved :: MIT License"],
      license="MIT",
      keywords="development testing",
      packages=find_packages(exclude=["tests"]),
      cmdclass=_CMD_CLASSES,
      install_requires=["setuptools", "green>=2.0.0"],
      extras_require={
          "polysquarelint": ["polysquare-setuptools-lint"],
          "green": ["mock", "testtools"],
          "upload": ["setuptools-markdown"]
      },
      entry_points={
          "distutils.commands": [
              "green=setuptools_green:GreenTestCommand",
          ]
      },
      zip_safe=True,
      include_package_data=True)
